# Copyright (C) 2020 by Vd.
# This file is part of BetterLogging, my logging improvement.
# BetterLogging is released under the MIT License (see LICENSE).


colors = dict(
    c_reset="\x1b[0m",
    c_bright="\x1b[1m",
    c_dim="\x1b[2m",
    c_underscore="\x1b[4m",
    c_blink="\x1b[5m",
    c_reverse="\x1b[7m",
    c_hidden="\x1b[8m",

    c_fg_black="\x1b[30m",
    c_fg_red="\x1b[31m",
    c_fg_green="\x1b[32m",
    c_fg_yellow="\x1b[33m",
    c_fg_blue="\x1b[34m",
    c_fg_magenta="\x1b[35m",
    c_fg_cyan="\x1b[36m",
    c_fg_white="\x1b[37m",
    c_fg_crimson="\x1b[38m",

    c_bg_black="\x1b[40m",
    c_bg_red="\x1b[41m",
    c_bg_green="\x1b[42m",
    c_bg_yellow="\x1b[43m",
    c_bg_blue="\x1b[44m",
    c_bg_magenta="\x1b[45m",
    c_bg_cyan="\x1b[46m",
    c_bg_white="\x1b[47m",
    c_bg_crimson="\x1b[48m",
)



# Copyright (C) 2020 by Vd.
# This file is part of BetterLogging, my logging improvement.
# BetterLogging is released under the MIT License (see LICENSE).


from os.path import join, dirname

import setuptools

setuptools.setup(
    name='betterlogging',
    version='0.0.7',
    author='Vd',
    author_email='vd@vd2.org',
    url='https://github.com/vd2org/betterlogging',
    license='MIT',
    description='My logging improvement',
    long_description=open(join(dirname(__file__), 'README.md')).read(),
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Utilities',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# Copyright (C) 2020 by Vd.
# This file is part of BetterLogging, my logging improvement.
# BetterLogging is released under the MIT License (see LICENSE).


from typing import TYPE_CHECKING

import logging
from logging import *  # noqa: F401, F403
from logging import root
from .colorized import ColorizedFormatter  # noqa: F401

from .trace import TRACE, TRACE_NAME


class BetterLogger(logging.Logger):
    def trace(self, msg, *args, **kwargs):
        if self.isEnabledFor(TRACE):
            self._log(TRACE, msg, args, **kwargs)


def trace(msg, *args, **kwargs):
    if len(logging.root.handlers) == 0:
        logging.basicConfig()
    logging.root.trace(msg, *args, **kwargs)  # noqa


logging.addLevelName(TRACE_NAME, TRACE)

logging.Logger.trace = BetterLogger.trace
logging.trace = trace
logging.TRACE = TRACE

if TYPE_CHECKING:
    Logger = BetterLogger
    root: Logger = root  # noqa


    def getLogger(name=None) -> 'Logger':  # noqa: E303
        return logging.getLogger(name)



# Copyright (C) 2020 by Vd.
# This file is part of BetterLogging, my logging improvement.
# BetterLogging is released under the MIT License (see LICENSE).


import logging
from logging import Formatter
from logging import LogRecord
from .outer.better_exceptions import ExceptionFormatter

from .colors import colors
from .trace import TRACE

DEFAULT_FORMAT = '%(c_fg_green)s%(asctime)s %(c_color)s%(levelname)-8s%(c_reset)s %(c_fg_cyan)s[%(name)s] ' \
                 '%(c_reset)s(%(filename)s:%(c_underscore)s%(funcName)s%(c_reset)s:%(lineno)d) ' \
                 '%(c_color)s%(message)s%(c_reset)s'

DEFAULT_LEVEL_COLORS = {
    TRACE: colors['c_fg_cyan'] + colors['c_bright'],
    logging.DEBUG: colors['c_fg_blue'] + colors['c_bright'],
    logging.INFO: colors['c_fg_white'] + colors['c_bright'],
    logging.WARN: colors['c_fg_yellow'] + colors['c_bright'],
    logging.ERROR: colors['c_fg_red'] + colors['c_bright'],
    logging.CRITICAL: colors['c_bg_red'] + colors['c_fg_white'] + colors['c_bright'],
    None: colors['c_reset']
}


class ColorizedFormatter(Formatter):
    def __init__(self, fmt=None, level_colors=None, *args, **kwargs):
        self.level_colors = level_colors or DEFAULT_LEVEL_COLORS
        fmt = fmt or DEFAULT_FORMAT
        self._formatter = ExceptionFormatter(colorize=True, backtrace=False, diagnose=True)

        super().__init__(fmt=fmt, *args, **kwargs)

    def format(self, record: LogRecord) -> str:
        color = self.level_colors.get(record.levelno, self.level_colors.get(None, ""))
        record.__dict__ = {**colors, "c_color": color, **record.__dict__}
        return super().format(record)

    def formatException(self, ei) -> str:
        ex = self._formatter.format_exception(*ei)
        return "".join(ex)
